/* libmpdclient
   (c) 2003-2019 The Music Player Daemon Project
   This project's homepage is: http://www.musicpd.org

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

   - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

   - Neither the name of the Music Player Daemon nor the names of its
   contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*! \file
 * \brief MPD client library
 *
 * Functions for sending command lists.
 *
 * Do not include this header directly.  Use mpd/client.h instead.
 */

#ifndef MPD_LIST_H
#define MPD_LIST_H

#include <stdbool.h>

struct mpd_connection;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Starts a command list, i.e. a group of pipelined commands which are
 * transferred in one block.  If one command fails, the rest of the
 * command list is canceled.
 *
 * Note that there is no guarantee on atomicity.
 *
 * @param connection the connection to MPD
 * @param discrete_ok tells MPD whether to acknowledge every list
 * command with an "list_OK" response
 * @return true on success
 */
bool
mpd_command_list_begin(struct mpd_connection *connection, bool discrete_ok);

/**
 * Commits the command list, i.e. makes MPD execute all commands which
 * were queued.
 *
 * Note: there is no way to cancel a command list once it is started.
 * You may however close the socket connection.
 *
 * @param connection the connection to MPD
 * @return true on success
 */
bool
mpd_command_list_end(struct mpd_connection *connection);

#ifdef __cplusplus
}
#endif

#endif
